/****************************************************************************
**
** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of the QtGui module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the 
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Digia.  
**
** If you have questions regarding the use of this file, please use
** contact form at http://qt.digia.com
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef QSYMBIANEVENT_H
#define QSYMBIANEVENT_H

#include <QtCore/qglobal.h>

#ifdef Q_OS_SYMBIAN

class TWsEvent;

QT_BEGIN_HEADER

QT_BEGIN_NAMESPACE

QT_MODULE(Gui)

class Q_GUI_EXPORT QSymbianEvent
{
public:
    enum Type {
        InvalidEvent,
        WindowServerEvent,
        CommandEvent,
        ResourceChangeEvent
    };

    QSymbianEvent(const TWsEvent *windowServerEvent);
    QSymbianEvent(Type eventType, int value);
    ~QSymbianEvent();

    Type type() const;
    bool isValid() const;

    const TWsEvent *windowServerEvent() const;
    int command() const;
    int resourceChangeType() const;

private:
    Type m_type;
    union {
        const void *m_eventPtr;
        int m_eventValue;

        qint64 m_reserved;
    };
};

inline QSymbianEvent::Type QSymbianEvent::type() const
{
    return m_type;
}

inline bool QSymbianEvent::isValid() const
{
    return m_type != InvalidEvent;
}

#ifndef QT_NO_DEBUG_STREAM
Q_GUI_EXPORT QDebug operator<<(QDebug dbg, const QSymbianEvent *o);
#endif

QT_END_NAMESPACE

QT_END_HEADER

#endif // Q_OS_SYMBIAN

#endif // QSYMBIANEVENT_H
